/************************
 * event.c
 * SDL port for Abuse
 * by Anthony Kruize
 ************************/

#include <SDL.h>

#include "system.h"
#include "image.hpp"
#include "palette.hpp"
#include "video.hpp"
#include "macs.hpp"
#include "mouse.hpp"
#include "event.hpp"
#include "timing.hpp"
#include "sprite.hpp"
#include "game.hpp"

//DKS -  new GP2X crapola:
#include "gp2x.h"
//DKS - these are the emulated cursor coordinates.  The real 0-319, 0-199 coordinates are
//	gotten by shifting each to the right 16.  They are like this so we can increment a bit
//	at a time in the input loop and get good responsiveness.
long priv_mousex = 160 << 16, priv_mousey = 100 << 16; // default is centered in screen
long priv_mouseaccel = 5;
long priv_mousevel	= 0;	// current mouse velocity
const long priv_mousemaxvel = 16000;

int priv_loadscreen = 0;	// are we at a file load/save screen?  if so, we display the
									// cursor and emulate the movement even when the game "state"
									// is running.
int gp2x_dpad[8] = {0, 0, 0, 0, 0, 0, 0, 0}; // holds current state of all DPAD "buttons"
int last_joy_dir = 0; // holds last direction of dpad, in degrees, right is 0 degrees, up is 90
int last_player_dir = 0;	// holds last player direction so he points towards direction last walked, 180 if left
int dpad_not_centered = 0;

extern int confirm_quit();
extern int get_key_binding( char *dir, int i );
extern int mouse_xscale, mouse_yscale;
short mouse_buttons[5] = { 0, 0, 0, 0, 0 };

// Pre-declarations
void handle_mouse( event &ev );

//
// Constructor
//
event_handler::event_handler( image *screen, palette *pal )
{
	CHECK( screen && pal );
	mouse = new JCMouse( screen, pal );
	mhere = mouse->exsist();
	last_keystat = get_key_flags();
	ewaiting = 0;

	// Ignore activate events
	SDL_EventState( SDL_ACTIVEEVENT, SDL_IGNORE );
}

//
// Destructor
//
event_handler::~event_handler()
{
	delete mouse;
}

//
// flush_screen()
// Redraw the screen
//
void event_handler::flush_screen()
{
	update_dirty( screen );
}

//
// get_key_flags()
// Return the flag for the key modifiers
//
int event_handler::get_key_flags()
{
	SDLMod key_flag;

	key_flag = SDL_GetModState();

	return ( ( key_flag & KMOD_SHIFT ) != 0 ) << 3 |
		   ( ( key_flag & KMOD_CTRL ) != 0 ) << 2 |
		   ( ( key_flag & KMOD_ALT ) != 0 ) << 1;
}

//DKS - changing for GP2X
////
//// event_waiting()
//// Are there any events in the queue?
////
//int event_handler::event_waiting()
//{
//	if( ewaiting )
//	{
//		return 1;
//	}
//	if( SDL_PollEvent( NULL ) )
//	{
//		ewaiting = 1;
//	}
//	return ewaiting;
//}
//
// event_waiting()
// Are there any events in the queue?
//
int event_handler::event_waiting()
{
	if( ewaiting )
	{
		return 1;
	}
	
	//DKS
	if (gp2x_input == INPUT_DPAD)
	{
		if (dpad_update())
		{
			// there was an emulated mouse cursor move/button-push
			ewaiting = 1;
			//			DKS - didn't help:
//			SDL_WarpMouse((short)(priv_mousex >> 16), (short)(priv_mousey >> 16));
		}
	}

	if( SDL_PollEvent( NULL ) )
	{
		ewaiting = 1;
	}
	

	return ewaiting;
}

//
// add_redraw()
// Add a redraw rectangle.
//
void event_handler::add_redraw( int X1, int Y1, int X2, int Y2, void *Start )
{
	event *ev;
	ev = new event;
	ev->type = EV_REDRAW;
	ev->redraw.x1 = X1;
	ev->redraw.x2 = X2;
	ev->redraw.y1 = Y1;
	ev->redraw.y2 = Y2;
	ev->redraw.start = Start;
	events.add_end( (linked_node *)ev );
}

//DKS - must change for GP2X
////
//// get_event()
//// Get and handle waiting events
////
//void event_handler::get_event( event &ev )
//{
//	event *ep;
//	while( !ewaiting )
//	{
//		event_waiting();
//
//		if (!ewaiting)
//			milli_wait( 1 );	// Sleep for 1 millisecond if there are no events
//	}
//
//	ep = (event *)events.first();
//	if( ep )
//	{
//		ev = *ep;
//		events.unlink( (linked_node *)ep );
//		delete ep;
//		ewaiting = events.first() != NULL;
//	}
//	else
//	{
//		// NOTE : that the mouse status should be known
//		// even if another event has occured.
//		ev.mouse_move.x = mouse->x();
//		ev.mouse_move.y = mouse->y();
//		ev.mouse_button = mouse->button();
//
//		// Gather events
//		SDL_Event event;
//		if( SDL_PollEvent( &event ) )
//		{
//			// always sort the mouse out
//			handle_mouse( ev );
//			mouse->update( ev.mouse_move.x, ev.mouse_move.y, ev.mouse_button );
//
//			switch( event.type )
//			{
//				case SDL_QUIT:
//				{
//					exit(0);
//					break;
//				}
//				case SDL_MOUSEBUTTONUP:
//				{
//					switch( event.button.button )
//					{
//						case 4:		// Mouse wheel goes up...
//						{
//							ev.key = get_key_binding( "b4", 0 );
//							ev.type = EV_KEYRELEASE;
//							break;
//						}
//						case 5:		// Mouse wheel goes down...
//						{
//							ev.key = get_key_binding( "b3", 0 );
//							ev.type = EV_KEYRELEASE;
//							break;
//						}
//					}
//					break;
//				}
//				case SDL_MOUSEBUTTONDOWN:
//				{
//					switch( event.button.button )
//					{
//						case 4:		// Mouse wheel goes up...
//						{
//							ev.key = get_key_binding( "b4", 0 );
//							ev.type = EV_KEY;
//							break;
//						}
//						case 5:		// Mouse wheel goes down...
//						{
//							ev.key = get_key_binding( "b3", 0 );
//							ev.type = EV_KEY;
//							break;
//						}
//					}
//					break;
//				}
//				case SDL_KEYDOWN:
//				case SDL_KEYUP:
//				{
//					// Default to EV_SPURIOUS
//					ev.key = EV_SPURIOUS;
//					if( event.type == SDL_KEYDOWN )
//					{
//						ev.type = EV_KEY;
//					}
//					else
//					{
//						ev.type = EV_KEYRELEASE;
//					}
//					switch( event.key.keysym.sym )
//					{
//						case SDLK_DOWN:			ev.key = JK_DOWN; break;
//						case SDLK_UP:			ev.key = JK_UP; break;
//						case SDLK_LEFT:			ev.key = JK_LEFT; break;
//						case SDLK_RIGHT:		ev.key = JK_RIGHT; break;
//						case SDLK_LCTRL:		ev.key = JK_CTRL_L; break;
//						case SDLK_RCTRL:		ev.key = JK_CTRL_R; break;
//						case SDLK_LALT:			ev.key = JK_ALT_L; break;
//						case SDLK_RALT:			ev.key = JK_ALT_R; break;
//						case SDLK_LSHIFT:		ev.key = JK_SHIFT_L; break;
//						case SDLK_RSHIFT:		ev.key = JK_SHIFT_R; break;
//						case SDLK_NUMLOCK:		ev.key = JK_NUM_LOCK; break;
//						case SDLK_HOME:			ev.key = JK_HOME; break;
//						case SDLK_END:			ev.key = JK_END; break;
//						case SDLK_BACKSPACE:	ev.key = JK_BACKSPACE; break;
//						case SDLK_TAB:			ev.key = JK_TAB; break;
//						case SDLK_RETURN:		ev.key = JK_ENTER; break;
//						case SDLK_SPACE:		ev.key = JK_SPACE; break;
//						case SDLK_CAPSLOCK:		ev.key = JK_CAPS; break;
//						case SDLK_ESCAPE:		ev.key = JK_ESC; break;
//						case SDLK_F1:			ev.key = JK_F1; break;
//						case SDLK_F2:			ev.key = JK_F2; break;
//						case SDLK_F3:			ev.key = JK_F3; break;
//						case SDLK_F4:			ev.key = JK_F4; break;
//						case SDLK_F5:			ev.key = JK_F5; break;
//						case SDLK_F6:			ev.key = JK_F6; break;
//						case SDLK_F7:			ev.key = JK_F7; break;
//						case SDLK_F8:			ev.key = JK_F8; break;
//						case SDLK_F9:			ev.key = JK_F9; break;
//						case SDLK_F10:			ev.key = JK_F10; break;
//						case SDLK_INSERT:		ev.key = JK_INSERT; break;
//						case SDLK_KP0:			ev.key = JK_INSERT; break;
//						case SDLK_PAGEUP:		ev.key = JK_PAGEUP; break;
//						case SDLK_PAGEDOWN:		ev.key = JK_PAGEDOWN; break;
//						case SDLK_KP8:			ev.key = JK_UP; break;
//						case SDLK_KP2:			ev.key = JK_DOWN; break;
//						case SDLK_KP4:			ev.key = JK_LEFT; break;
//						case SDLK_KP6:			ev.key = JK_RIGHT; break;
//						case SDLK_F11:
//						{
//							// Only handle key down
//							if( ev.type == EV_KEY )
//							{
//								// Toggle fullscreen
//								SDL_WM_ToggleFullScreen( SDL_GetVideoSurface() );
//							}
//							ev.key = EV_SPURIOUS;
//							break;
//						}
//						case SDLK_F12:
//						{
//							// Only handle key down
//							if( ev.type == EV_KEY )
//							{
//								// Toggle grab mouse
//								if( SDL_WM_GrabInput( SDL_GRAB_QUERY ) == SDL_GRAB_ON )
//								{
//									the_game->show_help( "Grab Mouse: OFF\n" );
//									SDL_WM_GrabInput( SDL_GRAB_OFF );
//								}
//								else
//								{
//									the_game->show_help( "Grab Mouse: ON\n" );
//									SDL_WM_GrabInput( SDL_GRAB_ON );
//								}
//							}
//							ev.key = EV_SPURIOUS;
//							break;
//						}
//						case SDLK_PRINT:	// print-screen key
//						{
//							// Only handle key down
//							if( ev.type == EV_KEY )
//							{
//								// Grab a screenshot
//								SDL_SaveBMP( SDL_GetVideoSurface(), "screenshot.bmp" );
//								the_game->show_help( "Screenshot saved to: screenshot.bmp.\n" );
//							}
//							ev.key = EV_SPURIOUS;
//							break;
//						}
//						default:
//						{
//							ev.key = (int)event.key.keysym.sym;
//							// Need to handle the case of shift being pressed
//							// There has to be a better way
//							if( (event.key.keysym.mod & KMOD_SHIFT) != 0 )
//							{
//								if( event.key.keysym.sym >= SDLK_a &&
//									event.key.keysym.sym <= SDLK_z )
//								{
//									ev.key -= 32;
//								}
//								else if( event.key.keysym.sym >= SDLK_1 &&
//										 event.key.keysym.sym <= SDLK_5 )
//								{
//									ev.key -= 16;
//								}
//								else
//								{
//									switch( event.key.keysym.sym )
//									{
//										case SDLK_6:
//											ev.key = SDLK_CARET; break;
//										case SDLK_7:
//										case SDLK_9:
//										case SDLK_0:
//											ev.key -= 17; break;
//										case SDLK_8:
//											ev.key = SDLK_ASTERISK; break;
//										case SDLK_MINUS:
//											ev.key = SDLK_UNDERSCORE; break;
//										case SDLK_EQUALS:
//											ev.key = SDLK_PLUS; break;
//										case SDLK_COMMA:
//											ev.key = SDLK_LESS; break;
//										case SDLK_PERIOD:
//											ev.key = SDLK_GREATER; break;
//										case SDLK_SLASH:
//											ev.key = SDLK_QUESTION; break;
//										case SDLK_SEMICOLON:
//											ev.key = SDLK_COLON; break;
//										case SDLK_QUOTE:
//											ev.key = SDLK_QUOTEDBL; break;
//										default:
//											break;
//									}
//								}
//							}
//						}
//					}
//					break;
//				}
//			}
//		}
//		// No more events
//		ewaiting = 0;
//	}
//}
//
// get_event()
// Get and handle waiting events
//
void event_handler::get_event( event &ev )
{
	if (gp2x_input == INPUT_MOUSE) 
	{
		//DKS - if using USB keyboard mouse, use original code here:
		event *ep;
		while( !ewaiting )
		{
			event_waiting();

			if (!ewaiting)
				milli_wait( 1 );	// Sleep for 1 millisecond if there are no events
		}

		ep = (event *)events.first();
		if( ep )
		{
			ev = *ep;
			events.unlink( (linked_node *)ep );
			delete ep;
			ewaiting = events.first() != NULL;
		}
		else
		{
			// NOTE : that the mouse status should be known
			// even if another event has occured.
			ev.mouse_move.x = mouse->x();
			ev.mouse_move.y = mouse->y();
			ev.mouse_button = mouse->button();

			// Gather events
			SDL_Event event;
			if( SDL_PollEvent( &event ) )
			{
				// always sort the mouse out
				handle_mouse( ev );
				mouse->update( ev.mouse_move.x, ev.mouse_move.y, ev.mouse_button );

				switch( event.type )
				{
					case SDL_QUIT:
					{
						exit(0);
						break;
					}
					case SDL_MOUSEBUTTONUP:
					{
						switch( event.button.button )
						{
							case 4:		// Mouse wheel goes up...
							{
								ev.key = get_key_binding( "b4", 0 );
								ev.type = EV_KEYRELEASE;
								break;
							}
							case 5:		// Mouse wheel goes down...
							{
								ev.key = get_key_binding( "b3", 0 );
								ev.type = EV_KEYRELEASE;
								break;
							}
						}
						break;
					}
					case SDL_MOUSEBUTTONDOWN:
					{
						switch( event.button.button )
						{
							case 4:		// Mouse wheel goes up...
							{
								ev.key = get_key_binding( "b4", 0 );
								ev.type = EV_KEY;
								break;
							}
							case 5:		// Mouse wheel goes down...
							{
								ev.key = get_key_binding( "b3", 0 );
								ev.type = EV_KEY;
								break;
							}
						}
						break;
					}
					case SDL_KEYDOWN:
					case SDL_KEYUP:
					{
						// Default to EV_SPURIOUS
						ev.key = EV_SPURIOUS;
						if( event.type == SDL_KEYDOWN )
						{
							ev.type = EV_KEY;
						}
						else
						{
							ev.type = EV_KEYRELEASE;
						}
						switch( event.key.keysym.sym )
						{
							case SDLK_DOWN:			ev.key = JK_DOWN; break;
							case SDLK_UP:			ev.key = JK_UP; break;
							case SDLK_LEFT:			ev.key = JK_LEFT; break;
							case SDLK_RIGHT:		ev.key = JK_RIGHT; break;
							case SDLK_LCTRL:		ev.key = JK_CTRL_L; break;
							case SDLK_RCTRL:		ev.key = JK_CTRL_R; break;
							case SDLK_LALT:			ev.key = JK_ALT_L; break;
							case SDLK_RALT:			ev.key = JK_ALT_R; break;
							case SDLK_LSHIFT:		ev.key = JK_SHIFT_L; break;
							case SDLK_RSHIFT:		ev.key = JK_SHIFT_R; break;
							case SDLK_NUMLOCK:		ev.key = JK_NUM_LOCK; break;
							case SDLK_HOME:			ev.key = JK_HOME; break;
							case SDLK_END:			ev.key = JK_END; break;
							case SDLK_BACKSPACE:	ev.key = JK_BACKSPACE; break;
							case SDLK_TAB:			ev.key = JK_TAB; break;
							case SDLK_RETURN:		ev.key = JK_ENTER; break;
							case SDLK_SPACE:		ev.key = JK_SPACE; break;
							case SDLK_CAPSLOCK:		ev.key = JK_CAPS; break;
							case SDLK_ESCAPE:		ev.key = JK_ESC; break;
							case SDLK_F1:			ev.key = JK_F1; break;
							case SDLK_F2:			ev.key = JK_F2; break;
							case SDLK_F3:			ev.key = JK_F3; break;
							case SDLK_F4:			ev.key = JK_F4; break;
							case SDLK_F5:			ev.key = JK_F5; break;
							case SDLK_F6:			ev.key = JK_F6; break;
							case SDLK_F7:			ev.key = JK_F7; break;
							case SDLK_F8:			ev.key = JK_F8; break;
							case SDLK_F9:			ev.key = JK_F9; break;
							case SDLK_F10:			ev.key = JK_F10; break;
							case SDLK_INSERT:		ev.key = JK_INSERT; break;
							case SDLK_KP0:			ev.key = JK_INSERT; break;
							case SDLK_PAGEUP:		ev.key = JK_PAGEUP; break;
							case SDLK_PAGEDOWN:		ev.key = JK_PAGEDOWN; break;
							case SDLK_KP8:			ev.key = JK_UP; break;
							case SDLK_KP2:			ev.key = JK_DOWN; break;
							case SDLK_KP4:			ev.key = JK_LEFT; break;
							case SDLK_KP6:			ev.key = JK_RIGHT; break;
							case SDLK_F11:
							{
								// Only handle key down
								if( ev.type == EV_KEY )
								{
									// Toggle fullscreen
									SDL_WM_ToggleFullScreen( SDL_GetVideoSurface() );
								}
								ev.key = EV_SPURIOUS;
								break;
							}
							case SDLK_F12:
							{
								// Only handle key down
								if( ev.type == EV_KEY )
								{
									// Toggle grab mouse
									if( SDL_WM_GrabInput( SDL_GRAB_QUERY ) == SDL_GRAB_ON )
									{
										the_game->show_help( "Grab Mouse: OFF\n" );
										SDL_WM_GrabInput( SDL_GRAB_OFF );
									}
									else
									{
										the_game->show_help( "Grab Mouse: ON\n" );
										SDL_WM_GrabInput( SDL_GRAB_ON );
									}
								}
								ev.key = EV_SPURIOUS;
								break;
							}
							case SDLK_PRINT:	// print-screen key
							{
								// Only handle key down
								if( ev.type == EV_KEY )
								{
									// Grab a screenshot
									SDL_SaveBMP( SDL_GetVideoSurface(), "screenshot.bmp" );
									the_game->show_help( "Screenshot saved to: screenshot.bmp.\n" );
								}
								ev.key = EV_SPURIOUS;
								break;
							}
							default:
							{
								ev.key = (int)event.key.keysym.sym;
								// Need to handle the case of shift being pressed
								// There has to be a better way
								if( (event.key.keysym.mod & KMOD_SHIFT) != 0 )
								{
									if( event.key.keysym.sym >= SDLK_a &&
										event.key.keysym.sym <= SDLK_z )
									{
										ev.key -= 32;
									}
									else if( event.key.keysym.sym >= SDLK_1 &&
											 event.key.keysym.sym <= SDLK_5 )
									{
										ev.key -= 16;
									}
									else
									{
										switch( event.key.keysym.sym )
										{
											case SDLK_6:
												ev.key = SDLK_CARET; break;
											case SDLK_7:
											case SDLK_9:
											case SDLK_0:
												ev.key -= 17; break;
											case SDLK_8:
												ev.key = SDLK_ASTERISK; break;
											case SDLK_MINUS:
												ev.key = SDLK_UNDERSCORE; break;
											case SDLK_EQUALS:
												ev.key = SDLK_PLUS; break;
											case SDLK_COMMA:
												ev.key = SDLK_LESS; break;
											case SDLK_PERIOD:
												ev.key = SDLK_GREATER; break;
											case SDLK_SLASH:
												ev.key = SDLK_QUESTION; break;
											case SDLK_SEMICOLON:
												ev.key = SDLK_COLON; break;
											case SDLK_QUOTE:
												ev.key = SDLK_QUOTEDBL; break;
											default:
												break;
										}
									}
								}
							}
						}
						break;
					}
				}
			}
			// No more events
			ewaiting = 0;
		}
	} else
	{
		// Not using USB keyboard/mouse, use modified code
		event *ep;
	//	while( !ewaiting )
	//	{
	//		event_waiting();
	//
	//		if (!ewaiting)
	//			milli_wait( 1 );	// Sleep for 1 millisecond if there are no events
	//
	//		//DKS - experiment that didn't help at all:
	////			handle_mouse( ev );
	////			mouse->update( ev.mouse_move.x, ev.mouse_move.y, ev.mouse_button );
	////			printf("ev.x:%d ev.y:%d ev.but:%d\n", ev.mouse_move.x, ev.mouse_move.y, ev.mouse_button);
	//	}

		//DKS - Initializing this part of the event fixed a lot of problems with mouse simulation
		ev.mouse_button = 0;

		//DKS - experiment
		ep = (event *)events.first();
		if( ep )
		{
			ev = *ep;
			events.unlink( (linked_node *)ep );
			delete ep;
			ewaiting = events.first() != NULL;
		}
		else
		{

	//		dpad_update();

			handle_mouse( ev );
			mouse->update( ev.mouse_move.x, ev.mouse_move.y, ev.mouse_button );

			// NOTE : that the mouse status should be known
			// even if another event has occured.
			ev.mouse_move.x = mouse->x();
			ev.mouse_move.y = mouse->y();
			ev.mouse_button = mouse->button();

//			//DKS - experiment
//			SDL_JoystickUpdate();
//			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_R) &&
//				SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L))
//			{
//				// L+R triggers acts as spacebar
//				ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
//				ev.key = JK_SPACE;
//			} else
//			{

			SDL_JoystickUpdate();	// Get very latest button states

			//Since button combos seem to cause problems with stuck keys, reset
			//them here
			if (!SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_R) &&
				!SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L))
			{
				the_game->set_key_down(JK_SPACE, 0);	
			}
			//Since button combos seem to cause problems with stuck keys, reset
			//them here
			if (!SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_SELECT) &&
				!SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_START))
			{
				the_game->set_key_down(JK_ESC, 0);	
			}

				// Gather events
				SDL_Event event;
				if( SDL_PollEvent( &event ) )
				{
					switch( event.type )
					{
						case SDL_QUIT:
						{
							//DKS - added these two lines:
							sound_uninit();
							close_graphics();

							exit(0);
							break;
						}
						case SDL_MOUSEBUTTONUP:
						{
							switch( event.button.button )
							{
								case 4:		// Mouse wheel goes up...
								{
									ev.key = get_key_binding( "b4", 0 );
										// special cases for our combo L+R to simulate spacebar 
	//									if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L) &&
	//										SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_R))
										if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L))
										{
											// L+R triggers acts as spacebar
											ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
											ev.key = JK_SPACE;
										}
									ev.type = EV_KEYRELEASE;
									break;
								}
								case 5:		// Mouse wheel goes down...
								{
									ev.key = get_key_binding( "b3", 0 );
									ev.type = EV_KEYRELEASE;
									break;
								}
							}
							break;
						}
						case SDL_MOUSEBUTTONDOWN:
						{
							switch( event.button.button )
							{
								case 4:		// Mouse wheel goes up...
								{
									ev.key = get_key_binding( "b4", 0 );
									ev.type = EV_KEY;
									break;
								}
								case 5:		// Mouse wheel goes down...
								{
									ev.key = get_key_binding( "b3", 0 );
									ev.type = EV_KEY;
									break;
								}
							}
							break;
						}
						//DKS - adding SDL GP2X joystick support
						case SDL_JOYBUTTONDOWN:
						case SDL_JOYBUTTONUP:
						{
							if (event.type == SDL_JOYBUTTONDOWN) {
								ev.key = EV_SPURIOUS;
							} else {
								ev.key = EV_SPURIOUS;
							}

							switch( event.jbutton.button )
							{
								case GP2X_BUTTON_SELECT:
								{ 
									ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
									if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_START))
									{
										// START & SELECT acts as Escape on GP2X 
										ev.key = JK_ESC;
									} else if (gp2x_input == INPUT_DPAD ||
													(gp2x_input == INPUT_TS && gp2x_ts_lefthanded)) 
									{
										// prev weapon 
										ev.key = JK_CTRL_R;
									}
									break;
								}
								case GP2X_BUTTON_START:
								{
									ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
									// special cases for our combo START+SELECT to escape game 
									if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_SELECT))
									{
										// START & SELECT acts as Escape on GP2X 
										ev.key = JK_ESC;
									} else if (gp2x_input == INPUT_DPAD ||
													(gp2x_input == INPUT_TS && gp2x_ts_lefthanded)) 
									{
										// next weapon
										ev.key = JK_INSERT;
									}
									break;
								}
								case GP2X_BUTTON_VOLDOWN:
								{
									if (( the_game->state == RUN_STATE ) &&
										(gp2x_input == INPUT_TS && !gp2x_ts_lefthanded))
									{	
										// prev weapn
										ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
										ev.key = JK_CTRL_R;
									}
									break;
								}
								case GP2X_BUTTON_VOLUP:
								{
									if (( the_game->state == RUN_STATE )  &&
										(gp2x_input == INPUT_TS && !gp2x_ts_lefthanded))
									{
										// next weapn
										ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
										ev.key = JK_INSERT;
									}
									break;
								}
								case GP2X_BUTTON_L:
								{
									//DKS - This method of doing a button combo to simulate a
									//keypress seems to work great for Escape, but for some
									//reason simulating spacebar does not work very well (it does
									//work, but sometimes takes a few presses.  I will still
									//simulate the spacebar here in case any parts of the
									//game still require it, but in cop.cpp I have it reading
									//these buttons directly instead of waiting for the spacebar
									//key event when the player dies.
									
									if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_R))
									{
										// L+R triggers acts as spacebar
										ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
										ev.key = JK_SPACE;
									}
									break;
								}
								case GP2X_BUTTON_R:
								{
									// see above
									if (gp2x_input != INPUT_MOUSE) 
									{
										if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L))
										{
											// L+R triggers acts as spacebar
											ev.type = (event.type == SDL_JOYBUTTONDOWN) ? EV_KEY : EV_KEYRELEASE;
											ev.key = JK_SPACE;
										}
									}
									break;
								}
								default:
									ev.key = EV_SPURIOUS;
							}
							break;
						}	
						case SDL_KEYDOWN:
						case SDL_KEYUP:
						{
							// Default to EV_SPURIOUS
							ev.key = EV_SPURIOUS;
							if( event.type == SDL_KEYDOWN )
							{
								ev.type = EV_KEY;
							}
							else
							{
								ev.type = EV_KEYRELEASE;
							}
							switch( event.key.keysym.sym )
							{
								case SDLK_DOWN:			ev.key = JK_DOWN; break;
								case SDLK_UP:			ev.key = JK_UP; break;
								case SDLK_LEFT:			ev.key = JK_LEFT; break;
								case SDLK_RIGHT:		ev.key = JK_RIGHT; break;
								case SDLK_LCTRL:		ev.key = JK_CTRL_L; break;
								case SDLK_RCTRL:		ev.key = JK_CTRL_R; break;
								case SDLK_LALT:			ev.key = JK_ALT_L; break;
								case SDLK_RALT:			ev.key = JK_ALT_R; break;
								case SDLK_LSHIFT:		ev.key = JK_SHIFT_L; break;
								case SDLK_RSHIFT:		ev.key = JK_SHIFT_R; break;
								case SDLK_NUMLOCK:		ev.key = JK_NUM_LOCK; break;
								case SDLK_HOME:			ev.key = JK_HOME; break;
								case SDLK_END:			ev.key = JK_END; break;
								case SDLK_BACKSPACE:	ev.key = JK_BACKSPACE; break;
								case SDLK_TAB:			ev.key = JK_TAB; break;
								case SDLK_RETURN:		ev.key = JK_ENTER; break;
								case SDLK_SPACE:		ev.key = JK_SPACE; break;
								case SDLK_CAPSLOCK:		ev.key = JK_CAPS; break;
								case SDLK_ESCAPE:		ev.key = JK_ESC; break;
								case SDLK_F1:			ev.key = JK_F1; break;
								case SDLK_F2:			ev.key = JK_F2; break;
								case SDLK_F3:			ev.key = JK_F3; break;
								case SDLK_F4:			ev.key = JK_F4; break;
								case SDLK_F5:			ev.key = JK_F5; break;
								case SDLK_F6:			ev.key = JK_F6; break;
								case SDLK_F7:			ev.key = JK_F7; break;
								case SDLK_F8:			ev.key = JK_F8; break;
								case SDLK_F9:			ev.key = JK_F9; break;
								case SDLK_F10:			ev.key = JK_F10; break;
								case SDLK_INSERT:		ev.key = JK_INSERT; break;
								case SDLK_KP0:			ev.key = JK_INSERT; break;
								case SDLK_PAGEUP:		ev.key = JK_PAGEUP; break;
								case SDLK_PAGEDOWN:		ev.key = JK_PAGEDOWN; break;
								case SDLK_KP8:			ev.key = JK_UP; break;
								case SDLK_KP2:			ev.key = JK_DOWN; break;
								case SDLK_KP4:			ev.key = JK_LEFT; break;
								case SDLK_KP6:			ev.key = JK_RIGHT; break;
								case SDLK_F11:
								{
									// Only handle key down
									if( ev.type == EV_KEY )
									{
										// Toggle fullscreen
										SDL_WM_ToggleFullScreen( SDL_GetVideoSurface() );
									}
									ev.key = EV_SPURIOUS;
									break;
								}
								case SDLK_F12:
								{
									// Only handle key down
									if( ev.type == EV_KEY )
									{
										// Toggle grab mouse
										if( SDL_WM_GrabInput( SDL_GRAB_QUERY ) == SDL_GRAB_ON )
										{
											the_game->show_help( "Grab Mouse: OFF\n" );
											SDL_WM_GrabInput( SDL_GRAB_OFF );
										}
										else
										{
											the_game->show_help( "Grab Mouse: ON\n" );
											SDL_WM_GrabInput( SDL_GRAB_ON );
										}
									}
									ev.key = EV_SPURIOUS;
									break;
								}
								case SDLK_PRINT:	// print-screen key
								{
									// Only handle key down
									if( ev.type == EV_KEY )
									{
										// Grab a screenshot
										SDL_SaveBMP( SDL_GetVideoSurface(), "screenshot.bmp" );
										the_game->show_help( "Screenshot saved to: screenshot.bmp.\n" );
									}
									ev.key = EV_SPURIOUS;
									break;
								}
								default:
								{
									ev.key = (int)event.key.keysym.sym;
									// Need to handle the case of shift being pressed
									// There has to be a better way
									if( (event.key.keysym.mod & KMOD_SHIFT) != 0 )
									{
										if( event.key.keysym.sym >= SDLK_a &&
											event.key.keysym.sym <= SDLK_z )
										{
											ev.key -= 32;
										}
										else if( event.key.keysym.sym >= SDLK_1 &&
												 event.key.keysym.sym <= SDLK_5 )
										{
											ev.key -= 16;
										}
										else
										{
											switch( event.key.keysym.sym )
											{
												case SDLK_6:
													ev.key = SDLK_CARET; break;
												case SDLK_7:
												case SDLK_9:
												case SDLK_0:
													ev.key -= 17; break;
												case SDLK_8:
													ev.key = SDLK_ASTERISK; break;
												case SDLK_MINUS:
													ev.key = SDLK_UNDERSCORE; break;
												case SDLK_EQUALS:
													ev.key = SDLK_PLUS; break;
												case SDLK_COMMA:
													ev.key = SDLK_LESS; break;
												case SDLK_PERIOD:
													ev.key = SDLK_GREATER; break;
												case SDLK_SLASH:
													ev.key = SDLK_QUESTION; break;
												case SDLK_SEMICOLON:
													ev.key = SDLK_COLON; break;
												case SDLK_QUOTE:
													ev.key = SDLK_QUOTEDBL; break;
												default:
													break;
											}
										}
									}
								}
							}
							break;
						}
					}
				}
				// No more events
				ewaiting = 0;

//			}
		}
	}
}

//DKS - Called in event loop.  Updates various globals based on dpad's current position.
//Returns 1 if emulated cursor was moved more than a pixel (> 65536) to signify 
//an equivalent to an SDL event took place (if dpad is held down, events won't otherwise
//continue to be generated, despite the need to show game some sort of event is occuring
int dpad_update(void)
{
	// we must keep track of last cursor position 
	static long last_mousex = 160;	
	static long last_mousey = 100;

	int ret = 0;

	//	DKS doing this in game.cpp's get_input()
//	SDL_JoystickUpdate(); 	//DKS - make sure we have latest state
	dpad_not_centered = 0;
	for (int i = 7; i >= 0; i--) 
	{
		gp2x_dpad[i] = SDL_JoystickGetButton(joy_gp2x, i);
//		if (gp2x_dpad[i]) {
//			printf("direction %d is pressed\n",i);
//		}
		dpad_not_centered += gp2x_dpad[i];
	}
	
//	// Handle new direct GP2X mixer setting via volume up/down buttons
//	if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_VOLDOWN))
//	{
//		if ((gp2x_input == INPUT_DPAD) ||
//			((gp2x_input == INPUT_TS) && (gp2x_ts_lefthanded)))
//		{
//			if (gp2x_mixervol > 15.0)
//			{
//				gp2x_mixervol -= 0.20;
//			} else
//			{
//				gp2x_mixervol -= 0.05;
//			}
//
//			if (gp2x_mixervol < 0)
//			{
//				gp2x_mixervol = 0;
//			}
//
//			gp2x_set_volume((int)gp2x_mixervol);
//
//			printf("volume: %d\n", (int)gp2x_mixervol);
//		}
//	} else if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_VOLUP))
//	{
//		if (gp2x_mixervol > 15.0)
//		{
//			gp2x_mixervol += 0.20;
//		} 
//		else
//		{
//			gp2x_mixervol += 0.05;
//		}
//
//		if (gp2x_mixervol > 100.0)
//		{
//			gp2x_mixervol = 100.0;
//		}
//		gp2x_set_volume((int)gp2x_mixervol);
//
//		printf("volume: %d\n", (int)gp2x_mixervol);
//	}

		


	if (( the_game->state == RUN_STATE ) && !priv_loadscreen)
	{
		// we are in the actual game	

		priv_mousevel = 0; // halt any possible emulated cursor movement
		
		if (dpad_not_centered) {
			if (gp2x_dpad[GP2X_BUTTON_UP]) {
				if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
					last_joy_dir = 112; 	// 112.5 rounded down for symmetry
				} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
					last_joy_dir = 68; 	// 67.5 rounded
				} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
					// only possible on a DPAD modded F100 
					last_joy_dir = 155; 	
				} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
					// only possible on a DPAD modded F100 
					last_joy_dir = 25; 	
				} else {
					last_joy_dir = 90;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_DOWN]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
					last_joy_dir = 248; 	// 247.5 rounded
				} else if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
					last_joy_dir = 292;	// 292.5 rounded down for symmetry
				} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
					// only possible on a DPAD modded F100
//					last_joy_dir = 225; 	
					last_joy_dir = 210; 	
				} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
					// only possible on a DPAD modded F100
//					last_joy_dir = 315; 	
					last_joy_dir = 330;
				} else {
					last_joy_dir = 270;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
	//						last_joy_dir = 203; 	//202.5 rounded
	//						last_joy_dir = 202; 	//202.5 rounded
					last_joy_dir = 185; 	//202.5 rounded
				} else if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
	//						last_joy_dir = 158;	//157.5 rounded 
					last_joy_dir = 175;
				} else {
					last_joy_dir = 180;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
	//						last_joy_dir = 337;	//337.5 rounded down for symmetry
	//						last_joy_dir = 338;	
					last_joy_dir = 355;	
				} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
	//						last_joy_dir = 23;	//22.5 rounded
	//						last_joy_dir = 22;	//22.5 rounded
					last_joy_dir = 5;	//22.5 rounded
				} else {
					last_joy_dir = 0;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
	//						last_joy_dir = 45;
//					last_joy_dir = 35;
				//		Adjusted this so firing up stairs is easier
					last_joy_dir = 25;
			} else if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
				//		Adjusted this so firing down stairs is easier
	//						last_joy_dir = 315;
					last_joy_dir = 330;
			} else if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
	//						last_joy_dir = 135;
//					last_joy_dir = 145;
				//		Adjusted this so firing up stairs is easier
					last_joy_dir = 155;
				
			} else if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
				//		Adjusted this so firing down stairs is easier
	//						last_joy_dir = 225;
					last_joy_dir = 210;
			} 
		} else {
			//dpad is centered
			last_joy_dir = last_player_dir;
		}
	} else
	{
		//DKS - we are in a GUI portion of the game

		if (!dpad_not_centered) 
		{
			// dpad is not being pushed in any direction, bring cursor movement to a halt
			priv_mousevel = 0;
		} else
		{
			// DPAD is being pushed in a direction, 
			// We are in a GUI portion of the game, we must emulate the mouse
			if (gp2x_dpad[GP2X_BUTTON_UP]) {
				if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
					priv_mousey -= priv_mousevel;
					priv_mousex -= (priv_mousevel >> 1); // we don't want to move left as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
					priv_mousey -= priv_mousevel;
					priv_mousex += (priv_mousevel >> 1); 	// we don't want to move right as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
					// only possible on a DPAD modded F100
					priv_mousey -= priv_mousevel;
					priv_mousex -= priv_mousevel; 	// we don't want to move right as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
					// only possible on a DPAD modded F100
					priv_mousey -= priv_mousevel;
					//DKS - bugfix here 9/7/08: below line was -= instead of +=
					priv_mousex += priv_mousevel; 	// we don't want to move right as fast if we're not pressing hard right
				} else {
					priv_mousey -= priv_mousevel;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_DOWN]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
					priv_mousey += priv_mousevel;
					priv_mousex -= (priv_mousevel >> 1); // we don't want to move left as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
					priv_mousey += priv_mousevel;
					priv_mousex += (priv_mousevel >> 1); 	// we don't want to move right as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
					// only possible on a DPAD modded F100
					priv_mousey += priv_mousevel;
					priv_mousex -= priv_mousevel; 	// we don't want to move right as fast if we're not pressing hard right
				} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
					// only possible on a DPAD modded F100
					//DKS - bugfix here 9/7/08: below line was -= instead of +=, line below that was also wrong
					priv_mousey += priv_mousevel;
					priv_mousex += priv_mousevel; 	// we don't want to move right as fast if we're not pressing hard right
				} else {
					priv_mousey += priv_mousevel;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_LEFT]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
					priv_mousey += (priv_mousevel >> 1);
					priv_mousex -= priv_mousevel; 
				} else if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
					priv_mousey -= (priv_mousevel >> 1);
					priv_mousex -= priv_mousevel; 
				} else {
					priv_mousex -= priv_mousevel;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_RIGHT]) {
				if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
					priv_mousey += (priv_mousevel >> 1);
					priv_mousex += priv_mousevel; 
				} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
					priv_mousey -= (priv_mousevel >> 1);
					priv_mousex += priv_mousevel; 
				} else {
					priv_mousex += priv_mousevel;
				}
			} else if (gp2x_dpad[GP2X_BUTTON_UPRIGHT]) {
					priv_mousex += priv_mousevel;
					priv_mousey -= priv_mousevel;
			} else if (gp2x_dpad[GP2X_BUTTON_DOWNRIGHT]) {
					priv_mousex += priv_mousevel;
					priv_mousey += priv_mousevel;
			} else if (gp2x_dpad[GP2X_BUTTON_UPLEFT]) {
					priv_mousex -= priv_mousevel;
					priv_mousey -= priv_mousevel;
				
			} else if (gp2x_dpad[GP2X_BUTTON_DOWNLEFT]) {
					priv_mousex -= priv_mousevel;
					priv_mousey += priv_mousevel;
			} 

			priv_mousevel += priv_mouseaccel;
			if (priv_mousevel > priv_mousemaxvel) {
				priv_mousevel = priv_mousemaxvel;
			}

			//DKS keep cursor within 1-pixel boundary of right and bottom edges
			if (priv_mousex > (318 << 16)) {
				priv_mousex = 318 << 16;
			} else if (priv_mousex < 0)
			{
				priv_mousex = 0;
			}

			if (priv_mousey > (198 << 16)) 
			{
				priv_mousey = 198 << 16;
			} else if (priv_mousey < 0) 
			{
				priv_mousey = 0;
			}

			if (labs(last_mousex - (priv_mousex >> 16)) > 0)
			{
				//debug
//				printf("Mouse moved 1 pixel\n");

////				SDL_WarpMouse((short)(priv_mousex >> 16), (short)(priv_mousey >> 16));
//				SDL_Event tmpevt;
//				tmpevt.type = SDL_MOUSEMOTION;
//				tmpevt.motion.type = SDL_MOUSEMOTION;
//				int tmpx, tmpy;
//				tmpevt.motion.state = priv_GetMouseState(&tmpx, &tmpy);			
//				tmpevt.motion.x = (short)tmpx;
//				tmpevt.motion.y = (short)tmpy;
//				tmpevt.motion.xrel = (priv_mousex >> 16) - last_mousex;
//				tmpevt.motion.yrel = (priv_mousey >> 16) - last_mousey;
//				SDL_PushEvent(&tmpevt);
				
				//emulated cursor has moved at least 1 screen pixel
				ret = 1;
			}
		}
	}

	return ret;
}
			

//DKS - this gets called instead of SDL_GetMouseState if we are using the DPAD of the GP2X, 
//i.e. running on an F100
Uint8 priv_GetMouseState( int *x, int *y)
{


	Uint8 tmp_but = 0;

	if (gp2x_input == INPUT_DPAD) 
	{
		if (( the_game->state != RUN_STATE ) || priv_loadscreen)
		{
			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_B))
			{
				tmp_but = SDL_BUTTON(1);
//				mouse_buttons[1] = 1;
			} else 
			{
//				tmp_but |= ~SDL_BUTTON(1);
				tmp_but = 0;
//				mouse_buttons[1] = 0;
			}
		}
//	}

			
//		if (( the_game->state == RUN_STATE ) && !priv_loadscreen) {
//			// we are not in a GUI portion of the game, so emulated left mouse button
//			// will be down if any dpad direction is currently pressed, or up if not
//			if (dpad_not_centered) 
//			{
//				tmp_but |= SDL_BUTTON(1);
//			}
//			
//			//emulated right mouse button will be held down if A is pressed
//			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_A))
//			{
//				tmp_but |= SDL_BUTTON(3);
//			}
//		} else 
//		{
////			printf("In GUI, Button B not pressed\n");
//			//we are in a GUI portion of the game, so emulated left mouse button
//			// will be down if B is pressed
//			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_B))
//			{
////				printf("In GUI, Button B pressed\n");
//				tmp_but |= SDL_BUTTON(1);
//			}
//		}	

			
//		if (( the_game->state == RUN_STATE ) || !priv_loadscreen)
//		{
//			if (dpad_not_centered)
//			{
//				// player is pressing dpad in a direction while in game, LMB is pressed.
//				tmp_but |= SDL_BUTTON(1);
////				if (!mouse_buttons[1]) 
////				{
////					ev.type = EV_MOUSE_BUTTON;
////					ev.key = EV_SPURIOUS;
////					mouse_buttons[1] = 1; // guessing that one is correct
////					ev.mouse_button |= LEFT_BUTTON;
////				} else 
////				{
////					ev.type = EV_SPURIOUS;
////					ev.key = EV_SPURIOUS;
////					ev.mouse_button |= LEFT_BUTTON;
////				}
////			} else
////			{
////				// dpad is centered while in the game, so LMB is unpressed
////				tmp_but &= ~SDL_BUTTON(1);
////				if (mouse_buttons[1]) {
////						ev.type = EV_MOUSE_BUTTON;
////						ev.key = EV_SPURIOUS;
////						mouse_buttons[1] = 0; // guessing that one is correct
////						ev.mouse_button |= LEFT_BUTTON;
////						ev.mouse_button &= ( 0xff - LEFT_BUTTON );
////				}
//			}
//			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_A))
//			{
//				//DKS - player is pressing button for special ability, so RMB is pressed.
//				tmp_but |= SDL_BUTTON(3);
////				if (!mouse_buttons[3]) 
////				{
////					ev.type = EV_MOUSE_BUTTON;
////					ev.key = EV_SPURIOUS;
////					mouse_buttons[3] = 1; // guessing that one is correct
////					ev.mouse_button |= RIGHT_BUTTON;
////				} else 
////				{
////					ev.type = EV_SPURIOUS;
////					ev.key = EV_SPURIOUS;
////					ev.mouse_button |= RIGHT_BUTTON;
////				}
//			}
//		} else
//	  	{
//			// On F100 and in GUI
//			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_B))
//			{
//				tmp_but |= SDL_BUTTON(1);
////				//DKS - player is pressing button for special ability, so RMB is pressed.
////				if (!mouse_buttons[1]) 
////				{
////					ev.type = EV_MOUSE_BUTTON;
////					ev.key = EV_SPURIOUS;
////					mouse_buttons[1] = 1; // guessing that one is correct
////					ev.mouse_button |= LEFT_BUTTON;
////				} else 
////				{
////					ev.type = EV_SPURIOUS;
////					ev.key = EV_SPURIOUS;
////					ev.mouse_button |= LEFT_BUTTON;
////				}
//			}
//		}
//
////
////		if (mouse_buttons[1]) 
////		{
////			tmp_but |= SDL_BUTTON(1);
////		} else
////		{	
////			tmp_but &= ~SDL_BUTTON(1);
////		}
////		if (mouse_buttons[2]) 
////		{
////			tmp_but |= SDL_BUTTON(2);
////		} else
////		{	
////			tmp_but &= ~SDL_BUTTON(2);
////		}
////		if (mouse_buttons[3]) 
////		{
////			tmp_but |= SDL_BUTTON(3);
////		} else
////		{	
////			tmp_but &= ~SDL_BUTTON(3);
////		}
//		if (mouse_buttons[1]) 
//		{
//			tmp_but |= SDL_BUTTON(1);
//		} else
//		{	
//			tmp_but &= ~SDL_BUTTON(1);
//		}
//		if (mouse_buttons[2]) 
//		{
//			tmp_but |= SDL_BUTTON(2);
//		} else
//		{	
//			tmp_but &= ~SDL_BUTTON(2);
//		}
//		if (mouse_buttons[3]) 
//		{
//			tmp_but |= SDL_BUTTON(3);
//		} else
//		{	
//			tmp_but &= ~SDL_BUTTON(3);
//		}

		if (x)
		{
			*x = priv_mousex >> 16;
		}

		if (y)
		{
			*y = priv_mousey >> 16;
		}
//		tmp_but = 0;	// buttons are handled in view.cpp


	} else if (gp2x_input == INPUT_TS)
	{
		// Touchscreen input mode
		tmp_but = SDL_GetMouseState( x, y );	

		if (( the_game->state != RUN_STATE ) || priv_loadscreen)
		{
			if (SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_L) ||
				 SDL_JoystickGetButton(joy_gp2x, GP2X_BUTTON_R))
			{
				tmp_but = SDL_BUTTON(1);
			} else 
			{
				tmp_but = 0;
			}
		}

	} else
	{
		// USB keyboard/mouse input mode
		// just keep the original Abuse-SDL behavior
		tmp_but = SDL_GetMouseState( x, y );
	}

	//DKS debug
//	printf("priv_GetMouseState() called x:%d y:%d but:%x\n", *x, *y, tmp_but);
	return tmp_but;
}

//DKS - altering for GP2X
////
//// Handle mouse motion and button presses
//// We don't handle the mousewheel here as
//// SDL_GetMouseState doesn't seem to be
//// able to detect that.
////
//void handle_mouse( event &ev )
//{
//	Uint8 buttons;
//	int x, y;
//
//	// always sort the mouse out
//	buttons = SDL_GetMouseState( &x, &y );
//	x = (x << 16) / mouse_xscale;
//	y = (y << 16) / mouse_yscale;
//	if( x > 319 )
//	{
//		x = 319;
//	}
//	if( y > 199 )
//	{
//		y = 199;
//	}
//	ev.mouse_move.x = x;
//	ev.mouse_move.y = y;
//	ev.type = EV_MOUSE_MOVE;
//
//	// Left button
//	if( (buttons & SDL_BUTTON(1)) && !mouse_buttons[1] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[1] = !mouse_buttons[1];
//		ev.mouse_button |= LEFT_BUTTON;
//	}
//	else if( !(buttons & SDL_BUTTON(1)) && mouse_buttons[1] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[1] = !mouse_buttons[1];
//		ev.mouse_button &= ( 0xff - LEFT_BUTTON );
//	}
//
//	// Middle button
//	if( (buttons & SDL_BUTTON(2)) && !mouse_buttons[2] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[2] = !mouse_buttons[2];
//		ev.mouse_button |= LEFT_BUTTON;
//		ev.mouse_button |= RIGHT_BUTTON;
//	}
//	else if( !(buttons & SDL_BUTTON(2)) && mouse_buttons[2] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[2] = !mouse_buttons[2];
//		ev.mouse_button &= ( 0xff - LEFT_BUTTON );
//		ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
//	}
//
//	// Right button
//	if( (buttons & SDL_BUTTON(3)) && !mouse_buttons[3] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[3] = !mouse_buttons[3];
//		ev.mouse_button |= RIGHT_BUTTON;
//	}
//	else if( !(buttons & SDL_BUTTON(3)) && mouse_buttons[3] )
//	{
//		ev.type = EV_MOUSE_BUTTON;
//		mouse_buttons[3] = !mouse_buttons[3];
//		ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
//	}
//}
//
// Handle mouse motion and button presses
// We don't handle the mousewheel here as
// SDL_GetMouseState doesn't seem to be
// able to detect that.
//
void handle_mouse( event &ev )
{
	if (gp2x_input == INPUT_MOUSE)
	{
		//If using USB keyboard/mouse, use original unmodified code:
		Uint8 buttons;
		int x, y;

		// always sort the mouse out
		buttons = SDL_GetMouseState( &x, &y );
		x = (x << 16) / mouse_xscale;
		y = (y << 16) / mouse_yscale;
		if( x > 319 )
		{
			x = 319;
		}
		if( y > 199 )
		{
			y = 199;
		}
		ev.mouse_move.x = x;
		ev.mouse_move.y = y;
		ev.type = EV_MOUSE_MOVE;

		// Left button
		if( (buttons & SDL_BUTTON(1)) && !mouse_buttons[1] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[1] = !mouse_buttons[1];
			ev.mouse_button |= LEFT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(1)) && mouse_buttons[1] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[1] = !mouse_buttons[1];
			ev.mouse_button &= ( 0xff - LEFT_BUTTON );
		}

		// Middle button
		if( (buttons & SDL_BUTTON(2)) && !mouse_buttons[2] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[2] = !mouse_buttons[2];
			ev.mouse_button |= LEFT_BUTTON;
			ev.mouse_button |= RIGHT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(2)) && mouse_buttons[2] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[2] = !mouse_buttons[2];
			ev.mouse_button &= ( 0xff - LEFT_BUTTON );
			ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
		}

		// Right button
		if( (buttons & SDL_BUTTON(3)) && !mouse_buttons[3] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[3] = !mouse_buttons[3];
			ev.mouse_button |= RIGHT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(3)) && mouse_buttons[3] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[3] = !mouse_buttons[3];
			ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
		}

	} else
	{
		// Not using USB keyboard/mouse, use modified code:
		Uint8 buttons;
		int x, y;

		ev.type = EV_MOUSE_MOVE;

		if (gp2x_input == INPUT_DPAD) 
		{
			dpad_update();
		}

		buttons = priv_GetMouseState( &x, &y);

		if( x > 319 )
		{
			x = 319;
		}
		if( y > 199 )
		{
			y = 199;
		}
		ev.mouse_move.x = x;
		ev.mouse_move.y = y;
		ev.type = EV_MOUSE_MOVE;

		// Left button
		if( (buttons & SDL_BUTTON(1)) && !mouse_buttons[1] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[1] = !mouse_buttons[1];
			ev.mouse_button |= LEFT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(1)) && mouse_buttons[1] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[1] = !mouse_buttons[1];
			ev.mouse_button &= ( 0xff - LEFT_BUTTON );
		}

		// Middle button
		if( (buttons & SDL_BUTTON(2)) && !mouse_buttons[2] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[2] = !mouse_buttons[2];
			ev.mouse_button |= LEFT_BUTTON;
			ev.mouse_button |= RIGHT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(2)) && mouse_buttons[2] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[2] = !mouse_buttons[2];
			ev.mouse_button &= ( 0xff - LEFT_BUTTON );
			ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
		}

		// Right button
		if( (buttons & SDL_BUTTON(3)) && !mouse_buttons[3] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[3] = !mouse_buttons[3];
			ev.mouse_button |= RIGHT_BUTTON;
		}
		else if( !(buttons & SDL_BUTTON(3)) && mouse_buttons[3] )
		{
			ev.type = EV_MOUSE_BUTTON;
			mouse_buttons[3] = !mouse_buttons[3];
			ev.mouse_button &= ( 0xff - RIGHT_BUTTON );
		}
	}
}
