/************************
 * setup.c
 * SDL port for Abuse
 * by Anthony Kruize
 ************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <signal.h>
#include <SDL.h>
#ifdef HAVE_OPENGL
#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#endif	/* __APPLE__ */
#endif	/* HAVE_OPENGL */
#include "specs.hpp"
#include "keys.hpp"
#include "setup.h"

//DKS
#include "gp2x.h"

flags_struct flags;
keys_struct keys;

extern unsigned int xres, yres;
static unsigned int scale;

//DKS - altering for GP2X
//
// Display help
//
//void showHelp()
//{
//	printf( "\n" );
//	printf( "Usage: abuse.sdl [options]\n" );
//	printf( "Options:\n\n" );
//	printf( "** Abuse Options **\n" );
//	printf( "  -size <arg>       Set the size of the screen\n" );
//	printf( "  -edit             Startup in editor mode\n" );
//	printf( "  -a <arg>          Use addon named <arg>\n" );
//	printf( "  -f <arg>          Load map file named <arg>\n" );
//	printf( "  -lisp             Startup in lisp interpreter mode\n" );
//	printf( "  -nodelay          Run at maximum speed\n" );
//	printf( "\n" );
//	printf( "** Abuse-SDL Options **\n" );
//	printf( "  -datadir <arg>    Set the location of the game data to <arg>\n" );
//	printf( "  -doublebuf        Enable double buffering\n" );
//	printf( "  -fullscreen       Enable fullscreen mode\n" );
//#ifdef HAVE_OPENGL
//	printf( "  -gl               Enable OpenGL\n" );
//	printf( "  -antialias        Enable anti-aliasing (with -gl only)\n" );
//#endif
//	printf( "  -h, --help        Display this text\n" );
//	printf( "  -mono             Disable stereo sound\n" );
//	printf( "  -nosound          Disable sound\n" );
//	printf( "  -scale <arg>      Scale to <arg>\n" );
////	printf( "  -x <arg>          Set the width to <arg>\n" );
////	printf( "  -y <arg>          Set the height to <arg>\n" );
//	printf( "\n" );
//	printf( "Anthony Kruize <trandor@labyrinth.net.au>\n" );
//	printf( "\n" );
//}
void showHelp()
{
	printf( "\n" );
	printf( "Usage: abuse.gpe [options]\n" );
	printf( "Options:\n\n" );
	printf( "** Abuse Options **\n" );
	printf( "  -size <arg>       Set the size of the screen\n" );
	printf( "  -edit             Startup in editor mode\n" );
	printf( "  -a <arg>          Use addon named <arg>\n" );
	printf( "  -f <arg>          Load map file named <arg>\n" );
	printf( "  -lisp             Startup in lisp interpreter mode\n" );
	printf( "  -nodelay          Run at maximum speed\n" );
	printf( "\n" );
	printf( "** Abuse-SDL Options **\n" );
//	printf( "  -datadir <arg>    Set the location of the game data to <arg>\n" );
//	printf( "  -doublebuf        Enable double buffering\n" );
//	printf( "  -fullscreen       Enable fullscreen mode\n" );
//#ifdef HAVE_OPENGL
//	printf( "  -gl               Enable OpenGL\n" );
//	printf( "  -antialias        Enable anti-aliasing (with -gl only)\n" );
//#endif
	printf( "  -h, --help        Display this text\n" );
//	printf( "  -mono             Disable stereo sound\n" );
	printf( "  -nosound          Disable sound\n" );
//	printf( "  -scale <arg>      Scale to <arg>\n" );
//	printf( "  -x <arg>          Set the width to <arg>\n" );
//	printf( "  -y <arg>          Set the height to <arg>\n" );
	printf( "\n");
	printf( "** GP2X-specific Options **\n");
	printf( "  -lefthanded       If enabled, touchscreen controls will be left-handed.\n");
	printf( "\n" );
	printf( "Dan Silsby (Senor Quack on gp32x.com): dansilsby <at> gmail <dot> com\n" );
	printf( "\n" );
}

//
// Create a default 'abuserc' file
//
void createRCFile( char *rcfile )
{
	FILE *fd = NULL;

	if( (fd = fopen( rcfile, "w" )) != NULL )
	{
		//DKS - changing for GP2X
//		fputs( "; Abuse-SDL Configuration file\n\n", fd );
//		fputs( "; Location of the datafiles\ndatadir=/var/games/abuse\n\n", fd );
//		fputs( "; Startup fullscreen\nfullscreen=0\n\n", fd );
//		fputs( "; Use DoubleBuffering\ndoublebuf=0\n\n", fd );
//		fputs( "; Use mono audio only\nmono=0\n\n", fd );
//		fputs( "; Grab the mouse to the window\ngrabmouse=0\n\n", fd );
//		fputs( "; Set the scale factor\nscale=2\n\n", fd );
//		fputs( "; Use OpenGL\ngl=0\n\n", fd );
//		fputs( "; Use anti-aliasing (with gl=1 only)\nantialias=1\n\n", fd );
////		fputs( "; Set the width of the window\nx=320\n\n", fd );
////		fputs( "; Set the height of the window\ny=200\n\n", fd );
//		fputs( "; Disable the SDL parachute in the case of a crash\nnosdlparachute=0\n\n", fd );
//		fputs( "; Key mappings\n", fd );
//		fputs( "left=LEFT\nright=RIGHT\nup=UP\ndown=DOWN\n", fd );
//		fputs( "fire=SPACE\nweapprev=CTRL_R\nweapnext=INSERT\n", fd );
		fputs( "; Abuse-SDL Configuration file\n\n", fd );
		fputs( "; Location of the datafiles\ndatadir=./data\n\n", fd );
		fputs( "; Startup fullscreen\nfullscreen=1\n\n", fd );
		fputs( "; Use DoubleBuffering\ndoublebuf=0\n\n", fd );
		fputs( "; Use mono audio only\nmono=0\n\n", fd );
		fputs( "; Grab the mouse to the window\ngrabmouse=0\n\n", fd );
		fputs( "; Set the scale factor\nscale=1\n\n", fd );
		fputs( "; Use OpenGL\ngl=0\n\n", fd );
		fputs( "; Use anti-aliasing (with gl=1 only)\nantialias=1\n\n", fd );
//		fputs( "; Set the width of the window\nx=320\n\n", fd );
//		fputs( "; Set the height of the window\ny=200\n\n", fd );
		fputs( "; Disable the SDL parachute in the case of a crash\nnosdlparachute=0\n\n", fd );
		fputs( "; Key mappings\n", fd );
		fputs( "left=LEFT\nright=RIGHT\nup=UP\ndown=DOWN\n", fd );
		fputs( "fire=SPACE\nweapprev=CTRL_R\nweapnext=INSERT\n", fd );

		fclose( fd );
	}
	else
	{
		printf( "Unable to create 'abuserc' file.\n" );
	}
}

//
// Read in the 'abuserc' file
//
void readRCFile()
{
	FILE *fd = NULL;
	char *rcfile;
	char buf[255];
	char *result;

	rcfile = (char *)jmalloc( strlen( get_save_filename_prefix() ) + 9, "rcfile" );
	sprintf( rcfile, "%s/abuserc", get_save_filename_prefix() );
	if( (fd = fopen( rcfile, "r" )) != NULL )
	{
		while( fgets( buf, sizeof( buf ), fd ) != NULL )
		{
			result = strtok( buf, "=" );
			if( strcasecmp( result, "fullscreen" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.fullscreen = atoi( result );
			}
			else if( strcasecmp( result, "doublebuf" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.doublebuf = atoi( result );
			}
			else if( strcasecmp( result, "mono" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.mono = atoi( result );
			}
			else if( strcasecmp( result, "grabmouse" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.grabmouse = atoi( result );
			}
			else if( strcasecmp( result, "scale" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				scale = atoi( result );
//				flags.xres = xres * atoi( result );
//				flags.yres = yres * atoi( result );
			}
/*			else if( strcasecmp( result, "x" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.xres = atoi( result );
			}
			else if( strcasecmp( result, "y" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.yres = atoi( result );
			}*/
			else if( strcasecmp( result, "gl" ) == 0 )
			{
				// We leave this in even if we don't have OpenGL so we can
				// at least inform the user.
				result = strtok( NULL, "\n" );
				flags.gl = atoi( result );
			}
#ifdef HAVE_OPENGL
			else if( strcasecmp( result, "antialias" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				if( atoi( result ) )
				{
					flags.antialias = GL_LINEAR;
				}
			}
#endif
			else if( strcasecmp( result, "nosdlparachute" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				flags.nosdlparachute = atoi( result );
			}
			else if( strcasecmp( result, "datadir" ) == 0 )
			{
				result = strtok( NULL, "\n" );
				set_filename_prefix( result );
			}
			else if( strcasecmp( result, "left" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.left = key_value( result );
			}
			else if( strcasecmp( result, "right" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.right = key_value( result );
			}
			else if( strcasecmp( result, "up" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.up = key_value( result );
			}
			else if( strcasecmp( result, "down" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.down = key_value( result );
			}
			else if( strcasecmp( result, "fire" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.b2 = key_value( result );
			}
			else if( strcasecmp( result, "special" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.b1 = key_value( result );
			}
			else if( strcasecmp( result, "weapprev" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.b3 = key_value( result );
			}
			else if( strcasecmp( result, "weapnext" ) == 0 )
			{
				result = strtok( NULL,"\n" );
				keys.b4 = key_value( result );
			}
		}
		fclose( fd );
	}
	else
	{
		// Couldn't open the abuserc file so let's create a default one
		createRCFile( rcfile );
	}
	jfree( rcfile );
}

//DKS - altering for GP2X
//
// Parse the command-line parameters
//
//void parseCommandLine( int argc, char **argv )
//{
//	for( int ii = 1; ii < argc; ii++ )
//	{
//		if( !strcasecmp( argv[ii], "-fullscreen" ) )
//		{
//			flags.fullscreen = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-doublebuf" ) )
//		{
//			flags.doublebuf = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-size" ) )
//		{
//			if( !sscanf( argv[++ii], "%d", &xres ) )
//			{
//				xres = 320;
//			}
//			if( !sscanf( argv[++ii], "%d", &yres ) )
//			{
//				yres = 200;
//			}
//		}
//		else if( !strcasecmp( argv[ii], "-scale" ) )
//		{
//			int result;
//			if( sscanf( argv[++ii], "%d", &result ) )
//			{
//				scale = result;
///*				flags.xres = xres * scale;
//				flags.yres = yres * scale;*/
//			}
//		}
///*		else if( !strcasecmp( argv[ii], "-x" ) )
//		{
//			int x;
//			if( sscanf( argv[++ii], "%d", &x ) )
//			{
//				flags.xres = x;
//			}
//		}
//		else if( !strcasecmp( argv[ii], "-y" ) )
//		{
//			int y;
//			if( sscanf( argv[++ii], "%d", &y ) )
//			{
//				flags.yres = y;
//			}
//		}*/
//		else if( !strcasecmp( argv[ii], "-nosound" ) )
//		{
//			flags.nosound = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-gl" ) )
//		{
//			// We leave this in even if we don't have OpenGL so we can
//			// at least inform the user.
//			flags.gl = 1;
//		}
//#ifdef HAVE_OPENGL
//		else if( !strcasecmp( argv[ii], "-antialias" ) )
//		{
//			flags.antialias = GL_LINEAR;
//		}
//#endif
//		else if( !strcasecmp( argv[ii], "-mono" ) )
//		{
//			flags.mono = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-datadir" ) )
//		{
//			char datadir[255];
//			if( sscanf( argv[++ii], "%s", datadir ) )
//			{
//				set_filename_prefix( datadir );
//			}
//		}
//		else if( !strcasecmp( argv[ii], "-h" ) || !strcasecmp( argv[ii], "--help" ) )
//		{
//			showHelp();
//			exit( 0 );
//		}
//	}
//}
void parseCommandLine( int argc, char **argv )
{
	for( int ii = 1; ii < argc; ii++ )
	{
//		if( !strcasecmp( argv[ii], "-fullscreen" ) )
//		{
//			flags.fullscreen = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-doublebuf" ) )
//		{
//			flags.doublebuf = 1;
//		}
//		else if( !strcasecmp( argv[ii], "-size" ) )
//		{
//			if( !sscanf( argv[++ii], "%d", &xres ) )
//			{
//				xres = 320;
//			}
//			if( !sscanf( argv[++ii], "%d", &yres ) )
//			{
//				yres = 200;
//			}
//		}
//		else if( !strcasecmp( argv[ii], "-scale" ) )
//		{
//			int result;
//			if( sscanf( argv[++ii], "%d", &result ) )
//			{
//				scale = result;
///*				flags.xres = xres * scale;
//				flags.yres = yres * scale;*/
//			}
//		}
/*		else if( !strcasecmp( argv[ii], "-x" ) )
		{
			int x;
			if( sscanf( argv[++ii], "%d", &x ) )
			{
				flags.xres = x;
			}
		}
		else if( !strcasecmp( argv[ii], "-y" ) )
		{
			int y;
			if( sscanf( argv[++ii], "%d", &y ) )
			{
				flags.yres = y;
			}
		}*/
//		else if( !strcasecmp( argv[ii], "-nosound" ) )
		if( !strcasecmp( argv[ii], "-nosound" ) )
		{
			flags.nosound = 1;
		}
//		else if( !strcasecmp( argv[ii], "-gl" ) )
//		{
//			// We leave this in even if we don't have OpenGL so we can
//			// at least inform the user.
//			flags.gl = 1;
//		}
//#ifdef HAVE_OPENGL
//		else if( !strcasecmp( argv[ii], "-antialias" ) )
//		{
//			flags.antialias = GL_LINEAR;
//		}
//#endif
//		else if( !strcasecmp( argv[ii], "-mono" ) )
//		{
//			flags.mono = 1;
//		}
		else if( !strcasecmp( argv[ii], "-datadir" ) )
		{
			char datadir[255];
			if( sscanf( argv[++ii], "%s", datadir ) )
			{
				set_filename_prefix( datadir );
			}
		}
		else if( !strcasecmp( argv[ii], "-h" ) || !strcasecmp( argv[ii], "--help" ) )
		{
			showHelp();
			exit( 0 );
		}
		else if( !strcasecmp( argv[ii], "-lefthanded" ) )
		{
			//DKS - new option for GP2X
			gp2x_ts_lefthanded = 1;
		}
		else if( !strcasecmp( argv[ii], "-input" ) )
		{
			//Pickle - new option for Wiz
			int y;
			if( sscanf( argv[++ii], "%d", &y ) )
			{
				printf( "Detected input selection %d\n", y );
				if( y <= 0 )
				    gp2x_input = INPUT_DPAD;
				else if( y == 1 )
				    gp2x_input = INPUT_TS;
				else if( y >= 2 )
				    gp2x_input = INPUT_MOUSE;
				else
				    gp2x_input = INPUT_DPAD;
			}
		}
	}
}

//
// Setup SDL and configuration
//
void setup( int argc, char **argv )
{
	// DKS - changing for GP2X
	// Initialise default settings
//	flags.fullscreen		= 0;			// Start in a window
//	flags.doublebuf			= 0;			// No double buffering
//	flags.mono				= 0;			// Enable stereo sound
//	flags.nosound			= 0;			// Enable sound
//	flags.grabmouse			= 0;			// Don't grab the mouse
//	flags.nosdlparachute	= 0;			// SDL error handling
//	flags.xres = xres		= 320;			// Default window width
//	flags.yres = yres		= 200;			// Default window height
//	flags.gl				= 0;			// Don't use opengl

	flags.fullscreen		= 1;			// Start in a window
	flags.doublebuf			= 0;			// No double buffering
	flags.mono				= 0;			// Enable stereo sound
	flags.nosound			= 0;			// Enable sound
	flags.grabmouse			= 0;			// Don't grab the mouse
	flags.nosdlparachute	= 0;			// SDL error handling
	flags.xres = xres		= 320;			// Default window width
	flags.yres = yres		= 200;			// Default window height
	flags.gl				= 0;			// Don't use opengl

#ifdef HAVE_OPENGL
	flags.antialias			= GL_NEAREST;	// Don't anti-alias
#endif
	keys.up					= key_value( "UP" );
	keys.down				= key_value( "DOWN" );
	keys.left				= key_value( "LEFT" );
	keys.right				= key_value( "RIGHT" );
	keys.b3					= key_value( "CTRL_R" );
	keys.b4					= key_value( "INSERT" );
	//DKS - changing for GP2X
//	scale					= 2;			// Default scale amount
	scale					= 1;			// Default scale amount

	// Display our name and version
	printf( "%s %s\n", PACKAGE, VERSION );

	// Initialize SDL with video and audio support
//	if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO ) < 0 )
//	if( SDL_Init(  SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0 )
	if( SDL_Init(  SDL_INIT_NOPARACHUTE | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0 )
	{
		printf( "Unable to initialise SDL : %s\n", SDL_GetError() );
		exit( 1 );
	}

	//DKS - we don't want this
//	atexit( SDL_Quit );

	//Remember what mixer was set to at program startup
	gp2x_mixervol = gp2x_mixervolatstartup = (float)gp2x_get_volume();
	fprintf(stderr,"GP2X: Sound mixer's original volume level detected:, %d\n", (int)gp2x_mixervolatstartup);

	//DKS - if I don't do this, for some reason I get distorted sound forever after launching
	//the program
	//Actually, now I think I might have only experienced this problem using the
	//cradle and USB keyboard/mouse.  Might have been caused by not having a 5V adapter
	//for the cradle.
	gp2x_set_volume((int)gp2x_mixervolatstartup);
	
	
	//DKS - initialize GP2X joystick
	joy_gp2x = SDL_JoystickOpen(0);

	// Set the savegame directory
	char *homedir;
	char *savedir;
	FILE *fd = NULL;

	//	DKS - changing for GP2X
//	if( (homedir = getenv( "HOME" )) != NULL )
//	{
//		savedir = (char *)jmalloc( strlen( homedir ) + 9, "savedir" );
//		sprintf( savedir, "%s/.abuse/", homedir );
//		// Check if we already have a savegame directory
//		if( (fd = fopen( savedir, "r" )) == NULL )
//		{
//			// FIXME: Add some error checking here
//			mkdir( savedir, S_IRUSR | S_IWUSR | S_IXUSR );
//		}
//		else
//		{
//			fclose( fd );
//		}
//		set_save_filename_prefix( savedir );
//		jfree( savedir );
//	}
//	else
//	{
//		// Warn the user that we couldn't set the savename prefix
//		printf( "WARNING: Unable to get $HOME environment variable.\n" );
//		printf( "         Savegames will probably fail.\n" );
//		// Just use the working directory.
//		// Hopefully they have write permissions....
//		set_save_filename_prefix( "" );
//	}

	//DKS - new code to allow launching anywhere on SD.  simple setting the save
	//prefix to "save/" was not enough (but it was for set_filename_prefix)
	
	char tmpbuf[300];
	char tmpbuf2[300];
	if (getcwd(tmpbuf, 300))
	{
		strcpy(tmpbuf2, tmpbuf);
		strcat(tmpbuf2, "/save/");
		set_save_filename_prefix( tmpbuf2 );
		fprintf(stderr, "Save folder is: %s\n", get_save_filename_prefix());
		strcpy(tmpbuf2, tmpbuf);
		strcat(tmpbuf2, "/data/");
		set_filename_prefix( tmpbuf2 );
		fprintf(stderr, "Data folder is: %s\n", get_filename_prefix());
		
	} else
	{
		fprintf(stderr, "Error getting detecting save folder, using default /mnt/sd/abusesave\n");
		set_save_filename_prefix( "/mnt/sd/abusesave" );
		fprintf(stderr, "Error getting detecting data folder, using default data/\n");
		set_filename_prefix( "data/" );
	 }
		
//	set_save_filename_prefix( "/mnt/smb/abuse/save/" );
//	set_save_filename_prefix( "/mnt/sd/abuse/save/" );

	// Set the datadir to a default value
	// (The current directory)
	
	//DKS - changing for GP2X:
//	set_filename_prefix( EXPDATADIR );
//	set_filename_prefix( "/mnt/smb/abuse/data/" );
//	set_filename_prefix( "/mnt/sd/abuse/data/" );
//	set_filename_prefix( "data/" );


	// Load the users configuration
	readRCFile();

	// Handle command-line parameters
	parseCommandLine( argc, argv );

	//DKS - gp2x_input's constant values are set in gp2x.h, we can tell if we are using
	//a dpad, usb mouse/keyboard, or f200 touchscreen by comparing it to INPUT_TS, INPUT_DPAD, 
	//or INPUT_MOUSE
	//	DKS - for some reason, this is getting set to INPUT_MOUSE on an F200
//	gp2x_input = SDL_GP2X_MouseType();
	//printf("DEBUG: SDL_GP2X_MouseType returned %d\n", SDL_GP2X_MouseType());
	printf("INPUT_DPAD:%d INPUT_TS:%d, INPUT_MOUSE:%d\n", INPUT_DPAD, INPUT_TS, INPUT_MOUSE);
	
	//gp2x_input = SDL_GP2X_MouseType();

	switch (gp2x_input) 
	{
		case INPUT_DPAD:
			printf("Using GP2X DPAD for input. No touchscreen or mice detected.\n");
			break;
		case INPUT_TS:
			printf("F200 detected. Using touchscreen for input.\n");
			break;
		case INPUT_MOUSE:
			printf("USB mouse detected.  Using USB mouse/keyboard input.\n");
			break;
		default:
			// shouldn't happen
			printf("Error detecting input type in setup.cpp.  Defaulting to DPAD input.\n");
	}

	// Calculate the scaled window size.
	//DKS - changing for GP2X
//	flags.xres = xres * scale;
//	flags.yres = yres * scale;
	flags.xres = xres = 320;
	flags.yres = yres = 200;

	// Stop SDL handling some errors
	if( flags.nosdlparachute )
	{
		// segmentation faults
		signal( SIGSEGV, SIG_DFL );
		// floating point errors
		signal( SIGFPE, SIG_DFL );
	}
}

//
// Get the key binding for the requested function
//
int get_key_binding( char *dir, int i )
{
	if( strcasecmp( dir, "left" ) == 0 )
	{
		return keys.left;
	}
	else if( strcasecmp( dir, "right" ) == 0 )
	{
		return keys.right;
	}
	else if( strcasecmp( dir, "up" ) == 0 )
	{
		return keys.up;
	}
	else if( strcasecmp( dir, "down" ) == 0 )
	{
		return keys.down;
	}
	else if( strcasecmp( dir, "b1" ) == 0 )
	{
		return keys.b1;
	}
	else if( strcasecmp( dir, "b2" ) == 0 )
	{
		return keys.b2;
	}
	else if( strcasecmp( dir, "b3" ) == 0 )
	{
		return keys.b3;
	}
	else if( strcasecmp( dir, "b4" ) == 0 )
	{
		return keys.b4;
	}
	return 0;
}
